"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _crypto = _interopRequireDefault(require("crypto"));

var Promise = _interopRequireWildcard(require("bluebird"));

var _memory = _interopRequireDefault(require("./session/memory"));

var _store = _interopRequireDefault(require("./session/store"));

var _cookie = _interopRequireDefault(require("./session/cookie"));

var _session = _interopRequireDefault(require("./session/session"));

var _utils = require("./session/utils");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const env = process.env.NODE_ENV;

const generateSessionId = () => _crypto.default.randomBytes(16).toString('hex');

const hash = sess => {
  const str = JSON.stringify(sess, (key, val) => {
    if (key === 'cookie') {
      return undefined;
    }

    return val;
  });
  return _crypto.default.createHash('sha1').update(str, 'utf8').digest('hex');
};

const session = (handler, options = {}) => {
  const name = options.name || 'sessionId';
  const cookieOptions = options.cookie || {};
  const store = options.store || new _memory.default();
  const generateId = options.generateId || generateSessionId;
  const touchAfter = options.touchAfter ? (0, _utils.parseToMs)(options.touchAfter) : 0;
  const rollingSession = options.rolling || false;
  const storePromisify = options.storePromisify || false;

  if (env === 'production' && store instanceof _memory.default) {
    console.warn('MemoryStore is not designed for production environment.');
  }

  if (typeof generateId !== 'function') throw new TypeError('generateId option must be a function');

  if (storePromisify) {
    store.get = Promise.promisify(store.get);
    store.set = Promise.promisify(store.set);
    store.destroy = Promise.promisify(store.destroy);
    if (typeof store.touch === 'function') store.touch = Promise.promisify(store.touch);
  }

  let storeReady = true;
  store.on('disconnect', () => {
    storeReady = false;
  });
  store.on('connect', () => {
    storeReady = true;
  });
  return (req, res) => {
    if (req.session) return handler(req, res);
    if (!storeReady) return handler(req, res);
    req.sessionStore = store;
    req.sessionId = req.cookies[name];

    const getSession = () => {
      if (!req.sessionId) {
        return Promise.resolve(hash(req.sessionStore.generate(req, generateId(), cookieOptions)));
      }

      return req.sessionStore.get(req.sessionId).then(sess => {
        if (sess) {
          return hash(req.sessionStore.createSession(req, sess));
        }

        return hash(req.sessionStore.generate(req, generateId(), cookieOptions));
      });
    };

    return getSession().then(hashedsess => {
      let sessionSaved = false;
      const oldEnd = res.end;

      res.end = function resEndProxy(...args) {
        const saveSession = () => {
          if (req.session) {
            if (hash(req.session) !== hashedsess) {
              sessionSaved = true;
              return req.session.save();
            }

            if (req.session.cookie.maxAge && touchAfter >= 0) {
              const minuteSinceTouched = req.session.cookie.maxAge - (req.session.cookie.expires - new Date());

              if (minuteSinceTouched < touchAfter) {
                return Promise.resolve();
              }

              return req.session.touch();
            }
          }

          return Promise.resolve();
        };

        return saveSession().then(() => {
          if ((req.cookies[name] !== req.sessionId || sessionSaved || rollingSession) && req.session) {
            res.setHeader('Set-Cookie', req.session.cookie.serialize(name, req.sessionId));
          }

          oldEnd.apply(this, args);
        });
      };

      return handler(req, res);
    });
  };
};

session.Store = _store.default;
session.Cookie = _cookie.default;
session.Session = _session.default;
session.MemoryStore = _memory.default;
var _default = session;
exports.default = _default;